#
# ZButtons
# (C) Copyright 2000 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
# All Rights Reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>
#

import App, Globals, OFS
import string
from Persistence import Persistent
from Products.ZSQLMethods.SQL import SQL
from Globals import HTMLFile, HTML
from Acquisition import Implicit
from OFS.Folder import Folder
from OFS.Image import getImageInfo
from App.Extensions import getObject
from webdav.NullResource import NullResource
from ZPublisher.HTTPResponse import HTTPResponse
import md5, base64
import sys
import cStringIO
import time

from urllib import quote
from PIL import Image, ImageDraw, ImageFont, ImageFilter, ImageChops

from Colors import COLOR, parse_color

def manage_addImageFactory(self, id, title='', REQUEST={}):
	""" Add a new Image Factory """

	obj=ImageFactory(id, title)
	self._setObject(id, obj)
	obj=getattr(self, id)
	if hasattr(obj, 'postInitialisation'):
		obj.postInitialisation()

	if REQUEST:
		return self.manage_main(self, REQUEST)
	return ''

manage_addImageFactoryForm=HTMLFile('manage_addImageFactoryForm',globals())
		
class ImageFactory(Persistent, Implicit, Folder):
	""" An image generation and delivery system """

	meta_type='Image Factory'
	manage_editImageFactoryForm=HTMLFile('manage_editImageFactoryForm',globals())
	manage_options=(
		{'label':'View', 'action':'manage_main'},
		{'label':'Cache Stuff', 'action':'manage_editImageFactoryForm'},
		)

	
	def __init__(self, id, title=''):
		self.id=id
		self.title=title
		self.cache={}
		self.hits=0
		self.cache_hits=0
		self.addSQLQueries()
		self.ids=0

	def addSQLQueries(self):
		pass

	def postInitialisation(self):
		pass

	sizes=['08', '10', '12', '14', '18', '24']
	fonts={'helvetica':'helv',
		   'symbol':'symb',
		   'lucinda':'lu',
		   'lucindatypewriter':'lut',
		   'lucindabright': 'lub',
		   'courier': 'cour',
		   'charter': 'char',
		   'times':'tim'}

	weights={'bold': 'B',
			 'regular': 'R',			 			 
			 'medium': ''}

	slants={'italic': 'I',
			'none':   '',
			'oblique': 'O'}
			 

	def manage_editImageFactory(self, cacheClearTime, REQUEST={}):
		""" Clear the Cache """

		if cacheClearTime > 0:
			now=int(time.time())
			chopchop=now-(int(cacheClearTime)*60)
			for x in self.cache.keys():
				if self.cache[x]['last_access'] < chopchop:
					self.manage_delObjects([self.cache[x]['id'],])
					del self.cache[x]
		if REQUEST:
			return self.manage_main(self, REQUEST)
		return ''
				
	def makeImage(self, options={}):

		self.hits=self.hits+1
		xs=0
		ys=0

		digest=md5.new()
		digest.update(str(options))
		digest=digest.digest()
		digest=string.strip(base64.encodestring(digest))

		if self.cache.has_key(digest):
			self.cache_hits=self.cache_hits+1
			self.cache[digest]['last_access']=time.time()
			self.cache[digest]['hits']=self.cache[digest]['hits']+1
			return self.absolute_url()+'/'+self.cache[digest]['id']

		sys.path.insert(100,SOFTWARE_HOME+'/Products/ZButtons/')
		if not self.fonts.has_key(options['face']):
			options['face']='times'

		if not self.weights.has_key(options['weight']):
			options['weight']='medium'

		if int(options['size']) < 0 or int(options['size']) > len(self.sizes):
			options['size']=2

		if not self.slants.has_key(options['slant']):
			options['slant']='regular'

		fontpath=''
		fontpath='BDF/%s%s%s%s.pil'%(
 			self.fonts[options['face']],
 			self.weights[options['weight']],
			self.slants[options['slant']],
			self.sizes[int(options['size'])])
		ifn = ImageFont.load_path(fontpath)
		alpha=None
		
		baseImage=None
		if options['bgimage']:
			content, width, height=getImageInfo(options['bgimage'])
			baseImage=Image.open(cStringIO.StringIO(options['bgimage']))
			baseImage=baseImage.convert("RGBA")
			if options['bwidth']:
				xs=int(options['bwidth'])
			else:
				xs= width

			if options['bheight']:
				ys=int(options['bheight'])
			else:
				ys=height


		else:
			tmpImage=Image.new("RGBA", (10,10), (0,0,0))
			tmpDraw=ImageDraw.Draw(tmpImage)
			xs,ys=tmpDraw.textsize(options['text'], font=ifn)
			xs=xs+6+int(options['xoffset'])+(2*(int(options['bevel'])))
			ys=ys+6+int(options['yoffset'])+(2*(int(options['bevel'])))
			if options['bwidth']:
				xs=int(options['bwidth'])
			if options['bheight']:
				ys=int(options['bheight'])
			del tmpImage
			del tmpDraw
			baseImage=Image.new("RGBA", (xs,ys),
									parse_color(options['bgcolour']))

		imsize=baseImage.size

		fontMask=Image.new("RGBA", imsize, (0,0,0))
		draw=ImageDraw.Draw(fontMask)
		sizex,sizey=draw.textsize(options['text'], font=ifn)
		bimsize=(sizex, sizey)
		width,height=imsize
		xoff=yoff=0
		draw.text((xoff,yoff), options['text'],
				  fill=(255,255,255),
				  font=ifn)

		fontMaskL=fontMask.convert("L")

		if options['shadow']:
			newMask=fontMask.filter(ImageFilter.SMOOTH).convert("L")
			shadowImage=Image.new("RGBA", imsize, parse_color(options['shadowcolour']))

			x=options['shadowoffset']+options['xoffset']
			y=options['shadowoffset']+options['yoffset']
			
			xxs, yys=bimsize
			if xxs > xs or yys > ys:
				shadowImage=shadowImage.resize((xs,ys), Image.BILINEAR)
				newMask=newMask.resize((xs, ys),Image.BILINEAR)
				box=(x, y, x+xs, y+ys)
			else:
				box=(x, y)
				
			baseImage.paste(shadowImage, box, newMask)
			del newMask
			del shadowImage

		if options['bevel']:
			bevelImage=None
			bevelImage=Image.new("RGBA", imsize, (0,0,0))
				
			bevel=int(options['bevel'])
			bevelDraw=ImageDraw.Draw(bevelImage)			

			del bevelDraw
			bevelMask=bevelImage.copy()
			bevelDraw=ImageDraw.Draw(bevelMask)			

			for i in range(bevel):
				step=float(128.0/float(width-(2*i)))
				shadColour=128.0
				for x in range(i,width-i):
					if not options['pressed']:
						bevelDraw.point((x,i),fill=(128-(shadColour-255),
													128-(shadColour-255),
													128-(shadColour-255)))

						bevelDraw.point((x,height-i-1), fill=(255-shadColour,
															  255-shadColour,
															  255-shadColour))
					else:
						bevelDraw.point((x,i),fill=(shadColour-128,
													shadColour-128,
													shadColour-128))
						bevelDraw.point((x,height-i-1), fill=(
							shadColour,shadColour, shadColour))
						
					shadColour=shadColour+step
					# Avoid roundoff.
					if shadColour > 255:
						shadColour=255
				step=float(128.0/float(height-(2*i)))
				shadColour=128.0
				for y in range(i,height-i):
					if not options['pressed']:
						bevelDraw.point((i,y), fill=(128-(shadColour-255),
													 128-(shadColour-255),
													 128-(shadColour-255)))
						bevelDraw.point((width-i-1, y), fill=(255-shadColour,
															  255-shadColour,
															  255-shadColour))


					else:
						bevelDraw.point((i,y), fill=(shadColour-128,
													 shadColour-128,
													 shadColour-128))
						bevelDraw.point((width-i-1, y), fill=(
							shadColour, shadColour, shadColour))
						
					shadColour=shadColour+step
					# Avoid roundoff.
					if shadColour >255:
						shadColour=255

			baseImage.paste(bevelImage, (0,0), bevelMask.convert("L"))
			del bevelDraw				
			del bevelImage
			del bevelMask

		# Paste this rectangle with the Font Mask should give us the 'coloured'
		# Font.
		fontImage=Image.new("RGBA", imsize, parse_color(options['fgcolour']))

		x=options['xoffset']
		y=options['yoffset']
		xxs, yys=bimsize
		
		if xxs > xs or yys > ys:
			fontImage=fontImage.resize((xs,ys), Image.BILINEAR)
			fontMaskL=fontMaskL.resize((xs,ys), Image.BILINEAR)
			box=(x, y, x+xs, y+ys)
		else:
			box=(x,y)
		
		baseImage.paste(fontImage, box, fontMaskL)
		finalImage=baseImage.copy()
		del baseImage
				
		# We have the image in hand...
		# Build the html template required...

		thisId=self.make_id()
		data=cStringIO.StringIO()
		try:
			finalImage.save(data,"JPEG")
		except:
			print "Toot my horn"			
			import traceback
			traceback.print_exc()

		self.manage_addButtonImage(thisId, data)
		self.cache[digest]={'id':thisId,
							'last_access':time.time(),
							'hits':1}

		# Trigger persistence machinery
		self.cache=self.cache
		self._p_changed=1
		return self.absolute_url()+'/'+thisId
	
	def make_id(self):
		next_id='img%012d'%(self.ids)
		self.ids=self.ids+1
		return next_id
	
	def manage_addButtonImage(self, imageId, data):
		content, width, height=getImageInfo(data)
		self._setObject(imageId, OFS.Image.Image(imageId, '', content, ''))
		imageObject=getattr(self, imageId)
		imageObject.manage_upload(file=data)
		
