#
# ZButtons
# (C) Copyright 2000 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
# All Rights Reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>
#

from DocumentTemplate.DT_Util import *
from DocumentTemplate.DT_String import String
from DateTime.DateTime import DateTime
from Globals import HTMLFile, HTML
import string
import re
import Acquisition

#
# <dtml-button>text</dtml-button>
#
# args:
#
# help         Print a help message about this tag
# literal      Do not parse the contents of the block before rendering
# fgcolour     The 'text' colour of the button
# bgcolour     The background colour of the button
# bgimage      An image to use instead of bgcolour
# xoffset      Start text at this xoffset
# yoffset      Start text at this yoffset
# shadow       Add a shadow
# shadowoffset Offset the shadow by this amount (-'s too)
# border       bordersize
# srconly      only return the src of the image, not the URL
# name         name parameter for the image

class ButtonTag:
	"""
	Dynamic Image Buttons
	"""

	
	name='button'
	blockContinuations=()

	_default_values={
		'help':         None,
		'literal':      None,
		'fgcolour':     '#ffffff',
		'bgcolour':     '#000000',
		'shadowcolour': '#000000',
		'bgimage':      None,
		'xoffset':      0,
		'yoffset':      0,
		'shadow':       0,
		'shadowoffset': 2,
		'border':       0,
		'size':         2,
		'face':         'charter',
		'weight':       'regular',
		'slant':        'none',
		'bwidth':       0,
		'bheight':      0,
		'width':        0,
		'height':       0,
		'bevel':        0,
		'pressed':      0
		}

	def __init__(self,blocks):

		tname, args, section = blocks[0]

		self.section=section
		self.args=args=parse_let_params(args)

		# I stole this from dtml-let...
		for i in range(len(args)):
			name,expr = args[i]
			if expr[:1]=='"' and expr[-1:]=='"' and len(expr) > 1:
				# expr shorthand
				expr=expr[1:-1]
				try: args[i] = name, Eval(expr).eval
				except SyntaxError, v:
					m,(huh,l,c,src) = v
					raise ParseError, (
						'<strong>Expression (Python) Syntax error</strong>:'
						'\n<pre>\n%s\n</pre>\n' % v[0],
						'let')
		
	def findImageFactory(self,root):

		while root is not None:
			if hasattr(root,'objectValues'):
				for o in root.objectValues():
					try:
						if (hasattr(o,'meta_type') and
							o.meta_type=='Image Factory'):
							return o
					except:
						pass
			if hasattr(root,'aq_parent'):
				root=root.aq_parent
			else:
				root=None
		return None
	ourHelpText=HTMLFile('helptext',globals())

	def render(self,md):
		""" Do the Translation """
		thisText=''
		us=md.this
		d={}
		md._push(d)
		try:
			for name,expr in self.args:
				if type(expr) is type(''):
					try:
						d[name]=md[expr]
					except:
						d[name]=expr
				else:
					d[name]=expr(md)
		finally:
			pass

		# Abort processing and return help text if they've asked for help.
		if d.has_key('help'):
			return render(us, self.ourHelpText)
			

		options={} # make sure we reset it

		for i in self._default_values.keys():
			if d.has_key(i):
				options[i]=d[i]
			else:
				options[i]=self._default_values[i]
				
		if d.has_key('bgimage'):
			#
			# us is an explicit aquirer, we need
			# an implicit acquirer..
			#
			we = us.__of__(us.aq_parent)
			
			bgimage=getattr(we,d['bgimage'])
			# bgimage=getattr(us,d['bgimage'])
			options['bgimage']=bgimage.data
		else:
			options['bgimage']=None


		# Do we render the sub-blocks first and then translate?
		
		if d.has_key('literal'):
			thisText=string.strip(self.section(us,md))
		else:
			thisText=string.strip(render_blocks(self.section.blocks,md))

		# Find the Image Factory
		_ImageMaker=self.findImageFactory(us)
		if not _ImageMaker:
			# If we can't find one, return the tag as-is
			return self.section(us,md)

		options['text']=thisText
		srconly=0
		
		if d.has_key('srconly'):
			srconly=1

		if not d.has_key('imgname'):
			d['imgname']=''

		# Get the image id

		imgHtml=''
		if not d.has_key('split'):
			if not d.has_key('href'):
				thisImg=_ImageMaker.makeImage(options)				
				imgHtml=self.makeHtmlTemplate(thisImg, d, d['imgname'], srconly,thisText)
			else:
				zbid=0				
				if not us.REQUEST.has_key('ZButtonJSInserted'):
					imgHtml="""<script>
					function ii(ri,hi,txt)
					{
					document.images[ri].src = hi.src;
					if( txt != 0 )
					setTimeout("top.window.status = '"+txt+"'", 100);
					}\n
					"""
					us.REQUEST['ZButtonJSInserted']=1
				else:
					imgHtml='<script>\n'
					
				if us.REQUEST.has_key('ZButtonJSId'):
					zbid=int(us.REQUEST['ZButtonJSId'])+1

				if d.has_key('bevel'):
					options['pressed']=1

				img1=_ImageMaker.makeImage(options)

				if d.has_key('link_bgcolour'):
					options['bgcolour']=d['link_bgcolour']
				if d.has_key('link_fgcolour'):
					options['fgcolour']=d['link_fgcolour']
				if d.has_key('link_bgimage'):
					#
					# us is an explicit aquirer, we need
					# an implicit acquirer..
					#
					we = us.__of__(us.aq_parent)

					link_bgimage=getattr(we,d['link_bgimage'])
					options['bgimage']=link_bgimage.data
				if d.has_key('bevel'):
					options['pressed']=1-int(options['pressed'])
					
				img2=_ImageMaker.makeImage(options)
				imgHtml=imgHtml+'ii%dl = new Image();ii%dl.src = "%s";\n'%(
					zbid,
					zbid,
					img1)
			
				imgHtml=imgHtml+'ii%dh = new Image();ii%dh.src = "%s";\n'%(
					zbid,
					zbid,
					img2)
				
				imgHtml=imgHtml+'</script>\n'
				us.REQUEST['ZButtonJSId']=zbid
				arefHtml=self.makeArefHtmlTemplate(zbid, 'ii', d['href'])
				d['border']=0
				imgHtml=imgHtml+arefHtml+self.makeHtmlTemplate(img1, d,
															   'ii%d'%(zbid),
															   0,
															   d['href'])
				imgHtml=imgHtml+'</a>'
		else:
			for word in string.split(thisText):
				options['text']=word
				thisImg=_ImageMaker.makeImage(options)

				# Split means no src only and no imageName
				tmpHtml=self.makeHtmlTemplate(thisImg, d, '', 0, word)
				imgHtml=imgHtml+tmpHtml
		return imgHtml

	def makeArefHtmlTemplate(self, imgId, jsFunc, url):
		#		htmlTemp='<a href="%s" onMouseover="%s(\'ii%d\', \'ii%dh\', \'%s\'); return true;" onMouseout="document.images[\'ii%d\'].src= ii%dl.src;">'%(
		#			url, jsFunc, imgId, imgId, url, imgId, imgId)

		htmlTemp='<a href="%s" onMouseover="document.images[\'ii%d\'].src= ii%dh.src;" onMouseout="document.images[\'ii%d\'].src= ii%dl.src;">'%(
			url, imgId, imgId, imgId, imgId)

		return htmlTemp
		
	def makeHtmlTemplate(self, thisImg, d, imagename, srconly, text):
		imgName=''
		border=''
		height=''
		width=''

		if srconly:
			return thisImg
		
		if imagename:
			imgName='name="%s"'%(imagename)

		if d.has_key('border'):
			border='border="%s"'%(d['border'])

		if d.has_key('height'):
			height='height="%s"'%(d['height'])

		if d.has_key('width'):
			width='width="%s"'%(d['width'])

		imgHtml='<img src="%s" %s %s %s %s alt="%s">'%(
			thisImg, imgName, border, height, width, text)

		return imgHtml		
        
	__call__=render


def parse_let_params(text,
					 result=None,
					 tag='button',
					 parmre=re.compile('([\000- ]*([^\000- ="]+)=([^\000- ="]+))'),
					 qparmre=re.compile('([\000- ]*([^\000- ="]+)="([^"]*)")'),
					 **parms):

	result=result or []
	mo = parmre.match(text)
	mo1= qparmre.match(text)

	if mo is not None:
		name=mo.group(2)
		value=mo.group(3)
		l=len(mo.group(1))
	elif mo1 is not None:
		name=mo1.group(2)
		value='"%s"' % mo1.group(3)
		l=len(mo1.group(1))
	else:
		if not text or not text.strip(): return result
		raise ParseError, ('invalid parameter: "%s"' % text, tag)

	result.append((name,value))

	text=text[l:].strip()
	if text: return apply(parse_let_params,(text,result,tag),parms)
	else: return result



String.commands['button'] = ButtonTag
	
